import os
from pathlib import Path
from apigateway.default import *


def gen_file(file_name: str):
    return Path(Path.cwd(), f"{file_name}.py")


def fill_default_content():
    with open(gen_file('settings'), mode='w', encoding='utf8') as f:
        f.write(f"{settings}")
    with open(gen_file('main'), mode='w', encoding='utf8') as f:
        f.write(f"{main}")
    with open(gen_file('urls'), mode='w', encoding='utf8') as f:
        f.write(f"{urlpatterns}")
    with open(gen_file('views'), mode='w', encoding='utf8') as f:
        f.write('')


def set_root_dir():
    """for prepare load settings"""
    os.environ['root'] = os.path.join(os.getcwd(), 'settings.py')


def load_settings():
    """load settings"""
    with open(os.environ.get('root'), encoding='utf8') as f:
        codes = f.read()[11:]
    res = eval(codes)
    return res


def startup():
    """create default file and fill it."""
    fill_default_content()
    set_root_dir()



settings = """settings = {
    "secret_key": "unknown",
    "iss": "Yingde"
}
"""

urlpatterns = """from views import *

urlpatterns=[]
"""

main = """from apigateway.event import Event
from apigateway.request import Request
from apigateway.response import Response

from urls import urlpatterns


def handler(event, context):
    # simulate flask request
    request = Request(Event(event), urlpatterns)
    # router
    flask_response = request.route()
    # turn flask response to api gateway response
    response = Response(flask_response)
    return response.response
"""



import pymysql
from pymysql.cursors import DictCursor
from apigateway.format import SqlStatement

# database setting
# db_host = environ.get('DBHOST', 'localhost')
# db_user = environ.get('DBUSER', 'test')
# db_pass = environ.get('DBPASS', 'test')
# db_name = environ.get('DBNAME', 'hawa')

db_host = "hawa2-django.cl5n467td4ui.rds.cn-northwest-1.amazonaws.com.cn"
db_user = "hawa2"
db_pass = "4oi2IMO48mz9"
db_name = "hawa_admin"


class DB:
    def __init__(self, host=db_host, db=db_name, usr=db_user, pwd=db_pass):
        self.err = None
        self.conn = self.make_connection(host, db, usr, pwd)
        self.cursor = self.conn.cursor()
        self.dict_cursor = self.conn.cursor(DictCursor)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    @staticmethod
    def make_connection(host, db, usr, pwd):
        return pymysql.connect(host=host, db=db, user=usr, passwd=pwd)

    def close(self):
        self.cursor.close()
        self.conn.close()

    def query(self, sql: SqlStatement):
        self.cursor.execute(sql)
        return self.cursor.fetchone()

    def query_dict(self, sql: SqlStatement):
        self.dict_cursor.execute(sql)
        return self.dict_cursor.fetchone()

    def query_many(self, sql: SqlStatement):
        self.cursor.execute(sql)
        return self.cursor.fetchall()

    def query_many_dict(self, sql: SqlStatement):
        self.dict_cursor.execute(sql)
        return self.dict_cursor.fetchall()

    def exe_and_commit(self, sql: SqlStatement):
        self.cursor.execute(sql)
        self.conn.commit()

    def insert(self, sql: SqlStatement):
        self.exe_and_commit(sql)

    def update(self, sql: SqlStatement):
        self.exe_and_commit(sql)

    def delete(self, sql: SqlStatement):
        self.exe_and_commit(sql)


class Case(DB):
    def __init__(self, host=db_host, db=db_name, usr=db_user, pwd=db_pass):
        super(Case, self).__init__(host=host, db=db, usr=usr, pwd=pwd)

    def query_user(self, username: str):
        """user:id,username,email,phone"""
        sql = f"SELECT id,username,email,phone FROM users WHERE username='{username}'" \
            f" AND role='A'"
        sql = SqlStatement(sql)
        user = self.query(sql)
        return user



import json


class Response:
    """turn flask restful response to api gateway response"""

    def __init__(self, data=None, headers=None, ):
        """
        :param data:flask response body
        """
        self.headers = headers if headers is not None else {}
        if "Access-Control-Allow-Origin" not in self.headers.keys():
            self.headers["Access-Control-Allow-Origin"] = '*'
        if "Access-Control-Allow-Headers" not in self.headers.keys():
            self.headers["Access-Control-Allow-Headers"] = '*'
        if isinstance(data, tuple):
            self.body = data[0]
            self.status_code = data[1]
        elif isinstance(data, dict):
            self.body = data
            self.status_code = 200

    @property
    def response(self):
        return {
            "statusCode": self.status_code,
            "headers": self.headers,
            "body": json.dumps(self.body)
        }



from datetime import datetime

import jwt

from apigateway.request import Request
from apigateway.settings import secret_key, db
from apigateway.utils import is_valid

now = datetime.now


class View:
    """here is the flask view, use absolute flask logic,request and response"""

    def __init__(self, request: Request = None):
        self.url = None
        self.request = request
        self.method = request.method
        self.response = None
        self.check_jwt()

    def dispatch(self, **kwargs):
        if not self.response:
            if self.method == 'GET':
                self.response = self.get(**kwargs)
            elif self.method == 'POST':
                self.response = self.post(**kwargs)
            elif self.method == 'PUT':
                self.response = self.put(**kwargs)
            elif self.method == 'DELETE':
                self.response = self.delete(**kwargs)
        return self.response

    def check_jwt(self):
        try:
            token = self.request.headers.get('Authorization')
        except AttributeError:
            self.response = {'error': 'without token'}, 401
            return None
        if is_valid(token):
            token = token[4:]
            try:
                payload = jwt.decode(token, secret_key, algorithm='HS256')
            except jwt.exceptions.ExpiredSignatureError:
                self.response = {'error': 'token is expired'}, 401
                return None
            current = now()
            exp = datetime.utcfromtimestamp(payload.get('exp'))
            username = payload.get('username')
            user = db.query_user(username)
            self.request.user = user
            if current < exp:
                # not expire
                pass
            else:
                # expire
                self.response = {'error': 'token is expired'}, 401
        else:
            self.response = {'error': f'wrong jwt:{token}'}, 401

    def get(self, *args, **kwargs):
        return NotImplemented

    def post(self, *args, **kwargs):
        return NotImplemented

    def delete(self, *args, **kwargs):
        return NotImplemented

    def put(self, *args, **kwargs):
        return NotImplemented



import json
import re

from apigateway.event import Event


class Request:
    """ turn api gateway request to flask request"""

    def __init__(self, event: Event, urlpatterns=None):
        """use lambda event to build flask format request"""
        # url is relative not absolute,different from flask
        self.event = event
        self.url = self.event.data.get("path")
        self.method = self.event.data.get("httpMethod", "GET")
        self.user = None
        self.headers = self.event.headers
        self.body = self.event.body
        self.path = {}
        self.args = self.event.data.get("queryStringParameters")
        self.urlpatterns = urlpatterns

    def get_data(self):
        """get data from request body,then turn it to json string"""
        return json.dumps(self.body)

    def route(self):
        """match urlpatterns and get flask response."""
        for path in self.urlpatterns:
            # match url
            router = re.match(path['path'], self.url)
            if router:
                self.path = router.groupdict()
                # pass request and path
                self.path['request'] = self
                flask_response = path['view'](self).dispatch(**self.path)
                break
        else:
            flask_response = {'error': 'something wrong in url'}, 400
        return flask_response



from setuptools import setup, find_packages

setup(name='flask-turn-apigateway',
      version='0.1.2',
      description='for develop apigateway',
      url='https://github.com/StevenLianaL/turnflaskrestfulapigateway',
      author='wz',
      author_email='wangzhou8284@163.cm',
      packages=find_packages(),
      zip_safe=False,
      python_requires='>=3.6',
      install_requires=['pyjwt', 'PyMySQL'],
      test_requires=[
      ])



import json


class Event:
    def __init__(self, event):
        self.event = event
        self.data = event
        self.prepare_event()

    def prepare_event(self):
        if isinstance(self.event, str):
            self.event = json.loads(self.event)

    @property
    def headers(self):
        headers = self.event.get('headers')
        if isinstance(headers, str):
            headers = json.loads(headers)
        return headers

    @property
    def body(self):
        body = self.event.get('body')
        if isinstance(body, str):
            body = json.loads(body)
        if isinstance(body, dict):
            if 'body' in body:
                body = body.get('body', {})
        return body
